<?PHP

/* Poidsy 0.3 - http://chris.smith.name/projects/poidsy
 * Copyright (c) 2008 Chris Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

 require_once(dirname(__FILE__) . '/poster.inc.php');
 require_once(dirname(__FILE__) . '/urlbuilder.inc.php');

 class KeyManager {

  private static $header = null;
  private static $data = null;

  private static function loadData() {
   if (self::$data == null) {
    $data = file(dirname(__FILE__) . '/keycache.php');
    self::$header = array_shift($data);
    self::$data = unserialize(implode("\n", $data));
   }
  }

  private static function saveData() {
   file_put_contents(dirname(__FILE__) . '/keycache.php', self::$header . serialize(self::$data));
  }

  public static function associate($server) {
   $data = URLBuilder::buildAssociate($server);

   try {
    $res = Poster::post($server, $data);
   } catch (Exception $ex) {
    return;
   }

   $data = array();

   foreach (explode("\n", $res) as $line) {
    if (preg_match('/^(.*?):(.*)$/', $line, $m)) {
     $data[$m[1]] = $m[2];
    }
   }

   $data['expires_at'] = time() + $data['expires_in'];

   self::$data[$server][$data['assoc_handle']] = $data;
   self::saveData();
  }

  public static function getHandle($server) {
   self::loadData();

   if (!isset(self::$data[$server])) {
    return null;
   }

   foreach (self::$data[$server] as $handle => $data) {
    if ($data['expires_at'] < time()) {
     unset(self::$data[$server][$handle]);
    } else {
     return $handle;
    }
   }

   return null;
  }

  public static function hasKey($server) {
   return self::getHandle($server) !== null;
  }

  public static function getData($server, $handle) {
   self::loadData();

   if (isset(self::$data[$server][$handle])) {
    if (self::$data[$server][$handle]['expires_at'] < time()) {
     self::removeKey($server, $handle);
     return null;
    } else {
     return self::$data[$server][$handle];
    }
   } else {
    return null;
   }
  }

  public static function authenticate($server, $args) {
   $data = self::getData($server, $args['openid_assoc_handle']);

   if ($data === null) {
    throw new Exception('No key available for that server/handle');
   }

   $contents = '';
   foreach (explode(',', $args['openid_signed']) as $arg) {
    $argn = str_replace('.', '_', $arg);
    $contents .= $arg . ':' . $args['openid_' . $argn] . "\n";
   }

   $sig = base64_encode(hash_hmac('sha1', $contents, base64_decode($data['mac_key']), true));

   if ($sig == $args['openid_sig']) {
    return true;
   } else {
    return false;
   }
  }

  public static function removeKey($server, $handle) {
   self::loadData();
   unset(self::$data[$server][$handle]);
   self::saveData();
  }

  public static function isSupported() {
   return @is_writable(dirname(__FILE__) . '/keycache.php')
	&& function_exists('hash_hmac');
  }

 }

?>
